#
# Simulation of Conway's "Game of Life".
#
# 2017, Tobias Kohn
#
from gturtle import *

CELLSIZE = 12

def render_line(y, line):
    """Draw all the points of a given line."""
    setPos(- len(line) * CELLSIZE / 2, y * CELLSIZE)
    heading(90)
    for item in line:
        if item == 1:
            setPenColor("black")
        else:
            setPenColor("white")
        forward(CELLSIZE)
    
def render_grid(grid):
    """Draw all the points of the entire grid."""
    for i in indices(grid):
        render_line(len(grid) // 2 + 1 - i, grid[i])

def update_grid(grid):
    """Update all points in the grid and recalculate which ones stay alive."""
    new_grid = []
    for j in indices(grid):
        line = grid[j]
        prev_line = grid[j-1]
        next_line = grid[(j+1) % len(grid)]
        new_line = [0] * len(line)
        for i in indices(line):
            k = (i+1) % len(line)
            count = (prev_line[i-1] + prev_line[i] + prev_line[k] +
                     line[i-1] +                     line[k] +
                     next_line[i-1] + next_line[i] + next_line[k])
            if (line[i] == 1) and (count in [2, 3]):
                new_line[i] = 1
            elif (line[i] == 0) and (count == 3):
                new_line[i] = 1
        new_grid.append(new_line)
    return new_grid

world = [[0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
         [0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
         [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]

# Initialize
makeTurtle()
hideTurtle()
setPenWidth(CELLSIZE)
setEndCap("clip")
# Update until a key is pressed
repeat:
    if getKeyCode() != 0:
        break
    render_grid(world)
    world = update_grid(world)
    delay(250)
dispose()